# Copyright (C) 2019 by E. Lücke

import ctypes as ct
import numpy as np
from PIL import Image


class GAPIWrapper:



    # 'Common_OpenInterface' prototypes
    proto_Common_OpenInterface = ct.CFUNCTYPE(ct.c_uint, ct.c_char_p, ct.c_void_p)
    proto_Common_GetFirmwareVersion = ct.CFUNCTYPE(ct.c_uint, ct.c_uint, ct.c_char_p, ct.c_void_p)
    proto_Lvds_Common_Config_Set_ByFile = ct.CFUNCTYPE(ct.c_uint, ct.c_uint, ct.c_uint, ct.c_char_p, ct.c_uint, ct.c_void_p)
    proto_Common_CloseInterface = ct.CFUNCTYPE(ct.c_uint, ct.c_uint)
    proto_Lvds_FrameGrabber_StartCapturing = ct.CFUNCTYPE(ct.c_uint, ct.c_uint)
    proto_Lvds_FrameGrabber_StopCapturing = ct.CFUNCTYPE(ct.c_uint, ct.c_uint)
    proto_Lvds_FrameGrabber_CaptureToBuffer = ct.CFUNCTYPE(ct.c_uint, ct.c_uint, ct.c_void_p, ct.c_void_p)


    # parameters of the call
    # (mode, name, default) mode: 1-in 2-out 3-in/out
    params_Common_OpenInterface = ((1, "interfaceName", 0), (1, "portHandle", 0))
    params_Common_GetFirmwareVersion = ((1, "portHandle", 0), (1, "response", 0), (1, "responseLength", 0))
    params_Lvds_Common_Config_Set_ByFile = ((1, "portHandle", 0), (1, "cmdFlags", 0), (1, "filename", 0), (1, "configurationSize", 0), (1, "configuration", 0))
    params_Common_CloseInterface = ((1, "portHandle", 0),)
    params_Lvds_FrameGrabber_StartCapturing = ((1, "portHandle", 0),)
    params_Lvds_FrameGrabber_StopCapturing = ((1, "portHandle", 0),)
    params_Lvds_FrameGrabber_CaptureToBuffer = ((1, "portHandle", 0), (1, "data", 0), (1, "length", 0))

    def __init__ (self, gapi_common_dll='g_api_common.dll', gapi_lvds_dll='g_api_lvds.dll'):

        #load the lib with the supplied path
        #GAPI uses C-Style function calls -> CDLL
        gapi_common_lib = ct.CDLL(gapi_common_dll)
        gapi_lvds_lib = ct.CDLL(gapi_lvds_dll)

        # load the function
        self.call_Common_OpenInterface = self.proto_Common_OpenInterface(('G_Common_OpenInterface', gapi_common_lib), self.params_Common_OpenInterface)
        self.call_Common_GetFirmwareVersion = self.proto_Common_GetFirmwareVersion(('G_Common_GetFirmwareVersion', gapi_common_lib), self.params_Common_GetFirmwareVersion)
        self.call_Lvds_Common_Config_Set_ByFile = self.proto_Lvds_Common_Config_Set_ByFile(('G_Lvds_Common_Config_Set_ByFile', gapi_lvds_lib), self.params_Lvds_Common_Config_Set_ByFile)
        self.call_Common_CloseInterface = self.proto_Common_CloseInterface(('G_Common_CloseInterface', gapi_common_lib), self.params_Common_CloseInterface)
        self.call_Lvds_FrameGrabber_StartCapturing = self.proto_Lvds_FrameGrabber_StartCapturing(('G_Lvds_FrameGrabber_StartCapturing', gapi_lvds_lib), self.params_Lvds_FrameGrabber_StartCapturing)
        self.call_Lvds_FrameGrabber_StopCapturing = self.proto_Lvds_FrameGrabber_StopCapturing(('G_Lvds_FrameGrabber_StopCapturing', gapi_lvds_lib), self.params_Lvds_FrameGrabber_StopCapturing)
        self.call_Lvds_FrameGrabber_CaptureToBuffer = self.proto_Lvds_FrameGrabber_CaptureToBuffer(('G_Lvds_FrameGrabber_CaptureToBuffer', gapi_lvds_lib), self.params_Lvds_FrameGrabber_CaptureToBuffer)


    def Common_OpenInterface(self, interfaceName):
        '''
        This funtion wraps the GAPI-Call Common_OpenInterface
        '''

        # create the parameters for the call in unmanaged c-memory
        intName = ct.c_char_p(interfaceName)
        portHandle = ct.c_uint(0)
        
        # call the previously loaded function
        # second parameter is pointer(byref) to unsigned long -> see gapi headers
        ret = self.call_Common_OpenInterface(intName, ct.byref(portHandle))

        port = portHandle.value

        return ret, port

    def Common_GetFirmwareVersion(self, port):
        '''
        This funtion wraps the GAPI-Call Common_GetFirmwareVersion
        '''

        resp = ct.create_string_buffer(4096)
        resplength = ct.c_uint(4096)
        portHandle = ct.c_uint(port)

        ret = self.call_Common_GetFirmwareVersion(portHandle, resp, ct.byref(resplength))

        versionString = resp.value.decode('utf-8')
        return ret, versionString

    def Common_CloseInterface(self, port):
        '''
        This funtion wraps the GAPI-Call Common_CloseInterface
        '''
        
        portHandle = ct.c_uint(port)
        
        ret = self.call_Common_CloseInterface(portHandle)

        return ret
    
    def Common_Config_Set_ByFile(self, port, filename, flag=0):
        '''
        This funtion wraps the GAPI-Call Lvds_Common_Config_Set_ByFile
        '''

        # create the parameters for the call in unmanaged c-memory
        portHandle = ct.c_uint(port)
        cmd_flag = ct.c_uint(flag)
        filename_c = ct.c_char_p(filename)
        
        configsize = ct.c_uint(0)
        config = ct.c_void_p(0)

        # call the previously loaded function
        ret = self.call_Lvds_Common_Config_Set_ByFile(portHandle, cmd_flag, filename_c, configsize, config)

        return ret

    def Lvds_FrameGrabber_StartCapturing(self, port):
        '''
        This funtion wraps the GAPI-Call Lvds_FrameGrabber_StartCapturing
        '''
        portHandle = ct.c_uint(port)

        ret = self.call_Lvds_FrameGrabber_StartCapturing(portHandle)

        return ret

    def Lvds_FrameGrabber_StopCapturing(self, port):
        '''
        This funtion wraps the GAPI-Call Lvds_FrameGrabber_StopCapturing
        '''
        portHandle = ct.c_uint(port)

        ret = self.call_Lvds_FrameGrabber_StopCapturing(portHandle)

        return ret

    def Lvds_FrameGrabber_CaptureToBuffer(self, port):
        '''
        This funtion wraps the GAPI-Call Lvds_FrameGrabber_CaptureToBuffer
        '''

        portHandle = ct.c_uint(port)
        data = ct.create_string_buffer(2800000)
        length = ct.c_uint(2800000)

        ret = self.call_Lvds_FrameGrabber_CaptureToBuffer(portHandle, data, ct.byref(length))

        return ret, data, length.value


def main():
    # make an instance of the Wrapper __init__ is called and loads the dll and functions
    gapi = GAPIWrapper()

    error = 0
    port = -1
    version = ''

    print('Open Interface...', end='')
    # call the python wrapped function
    error, port = gapi.Common_OpenInterface(b'LVDS3')
    print('[{}]'.format(error))
    print('opened interface:', port)
    

    print('Get Firmware Version...',end='')
    error, version = gapi.Common_GetFirmwareVersion(port)
    print('[{}]'.format(error))
    print('running version:',version)
    
    print('Load configuration file...',end='')
    error = gapi.Common_Config_Set_ByFile(port, b'C:\Users\elisa.luecke\Desktop\Python_Capture\VDS_Init4121_Grabber_ADV7611_v21_1280x720.xml')
    print('[{}]'.format(error))

    print('Start Capturing...', end='')
    error = gapi.Lvds_FrameGrabber_StartCapturing(port)
    print('[{}]'.format(error))
    print('started capturing')

    print('Capture to Buffer...', end='')
    error, data, frame_size = gapi.Lvds_FrameGrabber_CaptureToBuffer(port)
    print('[{}]'.format(error))

    # create matrix from image buffer
    img_raw = np.frombuffer(data, dtype=np.uint8, count=frame_size)
    img_raw = np.reshape(img_raw, (720,1280,3))
    # make rgb from bgr
    img_raw = img_raw[...,::-1]
    # create (pillow) image from matrix
    img = Image.fromarray(img_raw)

    #show frame
    img.show()
    # save frame 
    img.save('D:\\image.bmp')


    print('Stop Capturing...', end='')
    error = gapi.Lvds_FrameGrabber_StopCapturing(port)
    print('[{}]'.format(error))
    print('stopped capturing')

    print('Close Interface...', end='')
    error = gapi.Common_CloseInterface(port)
    print('[{}]'.format(error))
    print('closed interface:', port)

# main entry point of script
if __name__ == '__main__':
    main()
