# Copyright (C) 2019 by E. Luecke

# This example shows UART communication for Goepel LVDS device (Interface Name "LVDS1")
# For this example, a basicCON4121 with Media Interface Board MAX9276 is used
# Data to be sent: 0x79, 0x90, 0x05, 0x01, 0x19; expected response: 0xc3

import ctypes as ct

class GAPIWrapper:

    # 'Common_OpenInterface' prototypes
    proto_Common_OpenInterface = ct.CFUNCTYPE(ct.c_uint, ct.c_char_p, ct.c_void_p)
    proto_Common_CloseInterface = ct.CFUNCTYPE(ct.c_uint, ct.c_uint)
    proto_Lvds_Common_Data_Property_SetById = ct.CFUNCTYPE(ct.c_uint, ct.c_uint, ct.c_uint, ct.c_uint, ct.c_void_p)
    proto_Lvds_Common_Data_UartTransfer = ct.CFUNCTYPE(ct.c_uint, ct.c_uint, ct.c_void_p, ct.c_void_p)

    # parameters of the call
    # (mode, name, default) mode: 1-in 2-out 3-in/out
    params_Common_OpenInterface = ((1, "interfaceName", 0), (1, "portHandle", 0))
    params_Common_CloseInterface = ((1, "portHandle", 0),)
    params_Lvds_Common_Data_Property_SetById = ((1, "portHandle", 0), (1, "id", 0), (1, "length", 0), (1, "data", 0))
    params_Lvds_Common_Data_UartTransfer = ((1, "portHandle", 0), (1, "cmd", 0), (1, "rsp", 0))

    def __init__ (self, gapi_common_dll='g_api_common.dll', gapi_lvds_dll='g_api_lvds.dll'):

        #load the lib with the supplied path
        #GAPI uses C-Style function calls -> CDLL
        gapi_common_lib = ct.CDLL(gapi_common_dll)
        gapi_lvds_lib = ct.CDLL(gapi_lvds_dll)

        # load the function
        self.call_Common_OpenInterface = self.proto_Common_OpenInterface(('G_Common_OpenInterface', gapi_common_lib), self.params_Common_OpenInterface)
        self.call_Common_CloseInterface = self.proto_Common_CloseInterface(('G_Common_CloseInterface', gapi_common_lib), self.params_Common_CloseInterface)
        self.call_Lvds_Common_Data_Property_SetById = self.proto_Lvds_Common_Data_Property_SetById(('G_Lvds_Common_Data_Property_SetById', gapi_lvds_lib), self.params_Lvds_Common_Data_Property_SetById)
        self.call_Lvds_Common_Data_UartTransfer = self.proto_Lvds_Common_Data_UartTransfer(('G_Lvds_Common_Data_UartTransfer', gapi_lvds_lib), self.params_Lvds_Common_Data_UartTransfer)
    
    def Common_OpenInterface(self, interfaceName):
        '''
        This function wraps the GAPI-Call G_Common_OpenInterface
        '''

        # create the parameters for the call in unmanaged c-memory
        intName = ct.c_char_p(interfaceName)
        portHandle = ct.c_uint(0)
        
        # call the previously loaded function
        # second parameter is pointer(byref) to unsigned long -> see gapi headers
        ret = self.call_Common_OpenInterface(intName, ct.byref(portHandle))

        port = portHandle.value

        return ret, port

    def Common_CloseInterface(self, port):
        '''
        This function wraps the GAPI-Call G_Common_CloseInterface
        '''
        
        # create the parameters for the call in unmanaged c-memory
        portHandle = ct.c_uint(port)
        
         # call the previously loaded function
        ret = self.call_Common_CloseInterface(portHandle)

        return ret
    
    def Lvds_Common_Data_Property_SetById(self, port, propid, proplength, propdata):
        '''
        This function wraps the GAPI-Call G_Lvds_Common_Data_Property_SetById
        '''

        # create the parameters for the call in unmanaged c-memory
        portHandle = ct.c_uint(port)
        idx = ct.c_uint(propid)
        length = ct.c_uint(proplength)
        data = ct.create_string_buffer(propdata)
        data_p = ct.addressof(data)
        
        # call the previously loaded function
        ret = self.call_Lvds_Common_Data_Property_SetById(portHandle, idx, length, data_p)

        return ret

    def Lvds_Common_Data_UartTransfer(self, port, uartcmd, uartrsp):
        '''
        This function wraps the GAPI-Call G_Lvds_Common_Data_UartTransfer
        '''

        # create the parameters for the call in unmanaged c-memory
        # (struct-parameters will be defined in main function)
        portHandle = ct.c_uint(port)

        # call the previously loaded function
        ret = self.call_Lvds_Common_Data_UartTransfer(portHandle, ct.byref(uartcmd), ct.byref(uartrsp))

        return ret

class Lvds_Common_Data_UartTransfer_Cmd_t(ct.Structure):
    _fields_ = [
        ("Flags", ct.c_uint),
        ("NumberOfTxBytes", ct.c_char), # u_int8 is called char in c-world
        ("NumberOfRxBytes", ct.c_char),
        ("reserved1", ct.c_char),
        ("reserved2", ct.c_char),
        ("TxData", ct.c_char * 255)
    ]

class Lvds_Common_Data_UartTransfer_Rsp_t(ct.Structure):
    _fields_ = [
        ("Flags", ct.c_uint),
        ("NumberOfRxBytes", ct.c_char),
        ("reserved1", ct.c_char),
        ("reserved2", ct.c_char),
        ("reserved3", ct.c_char),
        ("RxData", ct.c_char * 2264)
    ]

def main():
    # make an instance of the Wrapper __init__ is called and loads the dll and functions
    gapi = GAPIWrapper()

    error = 0
    port = -1

    print('Open Interface...', end='')
    # call the python wrapped function
    error, port = gapi.Common_OpenInterface(b'LVDS1')
    print('[{}]'.format(error))
    print('opened interface:', port)

    print('Set Properties...', end='')
    error = gapi.Lvds_Common_Data_Property_SetById(port, 0, 1, b'\x07')
    print('[{}]'.format(error))
    error = gapi.Lvds_Common_Data_Property_SetById(port, 10, 4, b'\x00\xC2\x01\x00')
    print('[{}]'.format(error))
    error = gapi.Lvds_Common_Data_Property_SetById(port, 11, 1, b'\x01')
    print('[{}]'.format(error))

    print('Send UART command', end='')
    cmduart1 = Lvds_Common_Data_UartTransfer_Cmd_t(0, 5, 0, 0, 0, b'\x79\x90\x05\x01\x19')
    rspuart1 = Lvds_Common_Data_UartTransfer_Rsp_t(0, 0, 0, 0, 0, b'\x00')
    
    error = gapi.Lvds_Common_Data_UartTransfer(port, cmduart1, rspuart1)
    print('[{}]'.format(error))

    print('Read UART response', end='')
    cmduart2 = Lvds_Common_Data_UartTransfer_Cmd_t(1, 0, 1, 0, 0, b'\x00')
    rspuart2 = Lvds_Common_Data_UartTransfer_Rsp_t(0, 0, 0, 0, 0, b'\x01')

    error = gapi.Lvds_Common_Data_UartTransfer(port, cmduart2, rspuart2)
    print('[{}]'.format(error))
    print('Number of Rx Bytes:', rspuart2.NumberOfRxBytes)
    print('Rx Data:', rspuart2.RxData)

    print('Close Interface...', end='')
    error = gapi.Common_CloseInterface(port)
    print('[{}]'.format(error))
    print('closed interface:', port)

# main entry point of script
if __name__ == '__main__':
    main()

